#!/usr/bin/env python

import httplib
try:
	import simplejson as json
except ImportError:
	import json
from urllib import quote


class SurveyTown:
	def __init__(self, key, useHTTPS=False):
		self.key = key
		self.host = "api.myfeedback.io:80"
		self.path = "/2/REST/"
		self.version = "2.4.0"
		self.useHTTPS = useHTTPS
		if self.useHTTPS:
			self.host = "api.myfeedback.io:443"


	def Integration_Get_Cookie(self, clientId, username, time):
		'''
		Integrate SurveyTown into your own web application by setting your SURVEYS_SSO_COOKIE

		Required keyword arguments:

		clientId (int) - The client ID you are creating the cookie for, must be a client belonging to your reseller account
		username (string) - The belonging to the client ID you are logging in with the cookie
		time (int) - Cookie timeout in seconds

		Optional keyword arguments:


		Returns string - Cookie value to be set in SURVEYS_SSO_COOKIE
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Integration_Get_Cookie',
			'clientId': clientId,
			'username': username,
			'time': time,
		}

		data = self.makeCall(args)
		return data


	def Integration_Login_Get_Cookie(self, username, password, time):
		'''
		Integrate SurveyTown into your own web application by setting your SURVEYS_SSO_COOKIE using your client's credentials

		Required keyword arguments:

		username (string) - The of the client
		password (string) - The current of the client
		time (int) - Cookie timeout in seconds

		Optional keyword arguments:


		Returns struct - Returns a struct on a valid username/password combo
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Integration_Login_Get_Cookie',
			'username': username,
			'password': password,
			'time': time,
		}

		data = self.makeCall(args)
		return data


	def Response_Create(self, surveyId, updateParameters, optionalParameters=None, **kwargs):
		'''
		Create a survey response

		Required keyword arguments:

		surveyId (int) - The ID of the survey you want to respond to
		updateParameters (dict) - A dict of update parameters, any combination of which can be used
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		isComplete (bool) - Set a response as completed
		personCode (string) - A unique respondent identifier
		urlVariables (dict) - A dict of URL Variables: {name=>value}

		Returns int - Returns responseId on success
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'Response_Create',
			'surveyId': surveyId,
			'updateParameters': updateParameters,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def Response_Update(self, surveyId, responseId, updateParameters, optionalParameters=None, **kwargs):
		'''
		Update an existing survey response

		Required keyword arguments:

		surveyId (int) - The ID of the survey you want to update the response of
		responseId (int) - The ID of the response you want to update
		updateParameters (dict) - A dict of update parameters, any combination of which can be used
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		isComplete (bool) - Set a response as completed
		personCode (string) - A unique respondent identifier
		urlVariables (dict) - A dict of URL Variables: {name=>value}

		Returns int - Returns responseId on success
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'Response_Update',
			'surveyId': surveyId,
			'responseId': responseId,
			'updateParameters': updateParameters,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def Response_Delete(self, surveyId, responseId):
		'''
		Delete a response from a survey

		Required keyword arguments:

		surveyId (int) - The ID of the survey you want to delete from
		responseId (int) - The ID of the response you want to delete

		Optional keyword arguments:


		Returns bool - Returns true on success
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Response_Delete',
			'surveyId': surveyId,
			'responseId': responseId,
		}

		data = self.makeCall(args)
		return data


	def Survey_Start(self, surveyId):
		'''
		Start a survey

		Required keyword arguments:

		surveyId (int) - The ID of the survey you want to start

		Optional keyword arguments:


		Returns bool - Returns true on success
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Survey_Start',
			'surveyId': surveyId,
		}

		data = self.makeCall(args)
		return data


	def Survey_Stop(self, surveyId):
		'''
		Stop a survey

		Required keyword arguments:

		surveyId (int) - The ID of the survey you want to stop

		Optional keyword arguments:


		Returns bool - Returns true on success
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Survey_Stop',
			'surveyId': surveyId,
		}

		data = self.makeCall(args)
		return data


	def Survey_Copy(self, surveyId):
		'''
		Copy a survey

		Required keyword arguments:

		surveyId (int) - The ID of the survey you want to copy

		Optional keyword arguments:


		Returns int - Returns The ID of the new survey
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Survey_Copy',
			'surveyId': surveyId,
		}

		data = self.makeCall(args)
		return data


	def Survey_Delete(self, surveyId):
		'''
		Delete a survey

		Required keyword arguments:

		surveyId (int) - The ID of the survey you want to delete

		Optional keyword arguments:


		Returns bool - Returns true on success
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Survey_Delete',
			'surveyId': surveyId,
		}

		data = self.makeCall(args)
		return data


	def Survey_List(self):
		'''
		Get a listing of surveys

		Required keyword arguments:


		Optional keyword arguments:


		Returns struct - Array of surveys with the key of surveyId
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Survey_List',
		}

		data = self.makeCall(args)
		return data


	def Survey_Find(self, optionalParameters=None, **kwargs):
		'''
		Find surveys based on specific criteria
If the 'page' optionalParameter is provided, results are paginated to 1000 surveys per page.

		Required keyword arguments:

		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		surveyId (int) - Return only the survey with the supplied ID
		status (word) - Return only surveys matching this status. Must be one of "draft", "active", or "complete".
		type (word) - Return only surveys matching this type. Must be one of "survey" or "event".
		tagId (int) - Return only surveys having this tag ID
		startDate (date) - Return only surveys started on this date (UTC)
		endDate (date) - Return only surveys ending on this date (UTC)
		page (int) - Return surveys on this page of the search results
		offset (int) - Starting index to return results from. Indexed from 0.
		num (int) - Number of results to return

		Returns struct - Array of surveys with the key of surveyId and values of name, title, status, start_date, and end_date, ordered by survey ID ascending
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'Survey_Find',
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def Survey_Get_Info(self, surveyId):
		'''
		Retrieve a survey

		Required keyword arguments:

		surveyId (int) - The ID of the survey you want to retrieve

		Optional keyword arguments:


		Returns struct - Survey properties including questions, logic & actions
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Survey_Get_Info',
			'surveyId': surveyId,
		}

		data = self.makeCall(args)
		return data


	def Survey_Get_Questions(self, surveyId):
		'''
		Retrieves the questions for a particular survey

		Required keyword arguments:

		surveyId (int) - The ID of the survey you want to retrieve the questions for

		Optional keyword arguments:


		Returns struct - Array of survey questions, with the key of the question ID. The response format is similar to that of Survey_Get_Responses.
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Survey_Get_Questions',
			'surveyId': surveyId,
		}

		data = self.makeCall(args)
		return data


	def Survey_Get_Response_Count(self, surveyId):
		'''
		Counts how many responses a survey has.

		Required keyword arguments:

		surveyId (int) - ID of the survey you want to retrieve the response counts for

		Optional keyword arguments:


		Returns int - Count of how many responses there are for the given survey
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Survey_Get_Response_Count',
			'surveyId': surveyId,
		}

		data = self.makeCall(args)
		return data


	def Survey_Get_Responses_By_Email(self, emailAddress, optionalParameters=None, **kwargs):
		'''
		Retrieve responses a specific contact has entered

		Required keyword arguments:

		emailAddress (string) - The email address of the contact you want to retrieve responses for
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		surveyId (int) - Return only survey responses for this survey
		startDate (date) - Return only survey responses started on this date (UTC)
		endDate (date) - Return only survey responses completed on this date (UTC)
		is_complete (bool) - Return only survey responses that are complete (true) or incomplete (false)
		page (int) - Return questions on this page of the search results
		offset (int) - Starting index to return results from. Indexed from 0.
		num (int) - Number of results to return
		response_id (int) - Unique response identifier
		sortDir (string) - Sort direction, valid values: U, D

		Returns struct - Array of responses. See Survey_Get_Responses for response format documentation.
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'Survey_Get_Responses_By_Email',
			'emailAddress': emailAddress,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def Survey_Get_Responses(self, surveyId, optionalParameters=None, **kwargs):
		'''
		Retrieve responses for a specific survey
If the 'page' optionalParameter is provided, results are paginated to 1000 surveys per page.

		Required keyword arguments:

		surveyId (int) - The ID of the survey you want to retrieve the questions for
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		startDate (date) - Return only survey responses started on this date (UTC)
		endDate (date) - Return only survey responses completed on this date (UTC)
		completeDate (date) - Return only survey responses completed on or after this date (UTC)
		is_complete (bool) - Return only survey responses that are complete (true) or incomplete (false)
		page (int) - Return questions on this page of the search results
		offset (int) - Starting index to return results from. Indexed from 0.
		num (int) - Number of results to return
		response_id (int) - Unique response identifier
		contact_id (int) - Unique respondent indentifier created
		person_code (string) - Unique respondent indentifier provided
		sortDir (string) - Sort direction, valid values: U, D. Defaults to D.

		Returns struct - Array of surveys
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'Survey_Get_Responses',
			'surveyId': surveyId,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def Survey_Get_Responses_Zapier(self):
		'''
		Retrieve responses for a specific survey

		Required keyword arguments:


		Optional keyword arguments:


		Returns  - 
		'''
	
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'Survey_Get_Responses_Zapier',
		}

		data = self.makeCall(args)
		return data


	def Survey_Add_Tag(self, surveyId, tagId):
		'''
		Add a tag to a survey

		Required keyword arguments:

		surveyId (int) - The ID of the survey you want the tag added to
		tagId (int) - The ID of the tag you want to add

		Optional keyword arguments:


		Returns bool - Returns true on success
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Survey_Add_Tag',
			'surveyId': surveyId,
			'tagId': tagId,
		}

		data = self.makeCall(args)
		return data


	def Survey_Remove_Tag(self, surveyId, tagId):
		'''
		Remove a tag from a survey

		Required keyword arguments:

		surveyId (int) - The ID of the survey you want the tag removed from
		tagId (int) - The ID of the tag you want to remove

		Optional keyword arguments:


		Returns bool - Returns true on success
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Survey_Remove_Tag',
			'surveyId': surveyId,
			'tagId': tagId,
		}

		data = self.makeCall(args)
		return data


	def Tag_Create(self, name):
		'''
		Create a new tag

		Required keyword arguments:

		name (string) - The of your tag

		Optional keyword arguments:


		Returns int - id Returns the Tag ID of your new tag
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Tag_Create',
			'name': name,
		}

		data = self.makeCall(args)
		return data


	def Tag_Update(self, tagId, updateParameters):
		'''
		Update an existing Tag

		Required keyword arguments:

		tagId (int) - The of the tag you want to modify
		updateParameters (dict) - A dict of replacement values for your tag - only specify items that you want to change

		Optional keyword arguments:


		Returns bool - Returns true on success
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Tag_Update',
			'tagId': tagId,
			'updateParameters': updateParameters,
		}

		data = self.makeCall(args)
		return data


	def Tag_Delete(self, tagId):
		'''
		Delete a tag you created

		Required keyword arguments:

		tagId (int) - The Tag ID of the tag you wish to delete

		Optional keyword arguments:


		Returns bool - Returns true on success
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Tag_Delete',
			'tagId': tagId,
		}

		data = self.makeCall(args)
		return data


	def Tag_List(self, optionalParameters=None, **kwargs):
		'''
		Get a listing of tags

		Required keyword arguments:

		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		sortBy (string) - Field to sort by, value values: name, id
		sortDir (string) - Sort direction, valid values: U, D
		offset (int) - The number of records to skip (specify a starting point)
		num (int) - The maximum number of records to return

		Returns struct - Array of records with the key of tagId and values of tagId, name
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'Tag_List',
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data



	def http_build_query(self, params, topkey=''):
		
		if len(params) == 0:
			return ""
		
		result = ""
		
		if isinstance(params, dict):
			for key in params.keys():
				newkey = quote(key)
				if topkey != '':
					newkey = topkey + quote('[' + key + ']')
			
				if isinstance(params[key], dict):
					result += self.http_build_query(params[key], newkey)
			
				elif isinstance(params[key], list):
					i = 0
					for val in params[key]:
						if isinstance(val, dict):
							for subkey in val.keys():
								val[subkey] = str(val[subkey]).replace("'", "EC_SingleQuoteSentinelValue")
						result += newkey + quote('[' + str(i) + ']') + "=" + quote(str(val).replace('"', r'\"').replace("'",'"')) + "&"
						i = i + 1
			
				# boolean should have special treatment as well
				elif isinstance(params[key], bool):
					result += newkey + "=" + quote(str(int(params[key]))) + "&"
			
				# assume string (integers and floats work well)
				else:
					result += newkey + "=" + quote(str(params[key]).replace( "'", "EC_SingleQuoteSentinelValue" )) + "&"
		
		# remove the last '&'
		if result and topkey == '' and result[-1] == '&':
			result = result[:-1]
			
		result = result.replace("EC_SingleQuoteSentinelValue", "'")
		
		return result
	

	def makeCall(self, args):
		params = self.http_build_query(args)
		headers = {"Content-type": "application/x-www-form-urlencoded", "Accept": "text/plain", "User-Agent": "SurveyTown Python Wrapper " + self.version}
		
		if self.useHTTPS:
			conn = httplib.HTTPSConnection(self.host)
		else:
			conn = httplib.HTTPConnection(self.host)
		conn.request("POST", self.path, params, headers)
		
		response = conn.getresponse()
		jsonstr = response.read()
		
		data = None
		data = json.loads(jsonstr)
		
		if data is None:
			raise Exception("Could not fetch data from the API.")

		if isinstance(data, dict) and 'result' in data and data['result'] == "error":
			raise Exception("API Error: "+data['message']+" ("+repr(data['code'])+")")
		
		return data

		
class SurveyTown_Reseller(SurveyTown):

